=== modified file 'HACKING'
--- HACKING	2005-11-24 04:30:18 +0000
+++ HACKING	2005-12-16 00:17:28 +0000
@@ -247,4 +247,48 @@
     Revision 0 is always the null revision; others are 1-based
     indexes into the branch's revision history.
 
-:: vim: tw=74 ai
+
+Merge/review process
+====================
+
+If you'd like to propose a change, please post to the
+bazaar-ng@lists.canonical.com list with a patch, bzr changeset, or link to a
+branch.  Please put '[patch]' in the subject so we can pick them out, and
+include some text explaining the change.  Remember to put an update to the NEWS
+file in your diff, if it makes any changes visible to users or plugin
+developers.  Please include a diff against mainline if you're giving a link to
+a branch.
+
+Please indicate if you think the code is ready to merge, or if it's just a
+draft or for discussion.  If you want comments from many developers rather than
+to be merged, you can put '[rfc]' in the subject lines.
+
+Anyone is welcome to review code.  There are broadly three gates for
+code to get in:
+
+ * Doesn't reduce test coverage: if it adds new methods or commands,
+   there should be tests for them.  There is a good test framework
+   and plenty of examples to crib from, but if you are having trouble
+   working out how to test something feel free to post a draft patch
+   and ask for help.
+
+ * Doesn't reduce design clarity, such as by entangling objects
+   we're trying to separate.  This is mostly something the more
+   experienced reviewers need to help check.
+
+ * Improves bugs, features, speed, or code simplicity.
+
+Code that goes in should pass all three.
+
+If you read a patch please reply and say so.  We can use a numeric scale
+of -1, -0, +0, +1, meaning respectively "really don't want it in current
+form", "somewhat uncomfortable", "ok with me", and "please put it in".
+Anyone can "vote".   (It's not really voting, just a terse expression.)
+
+If something gets say two +1 votes from core reviewers, and no
+vetos, then it's OK to come in.  Any of the core developers can bring it
+into their integration branch, which I'll merge regularly.  (If you do
+so, please reply and say so.)
+
+
+:: vim:tw=74:ai

=== modified file 'Makefile'
--- Makefile	2005-12-02 05:25:53 +0000
+++ Makefile	2005-12-12 23:14:43 +0000
@@ -2,6 +2,8 @@
 
 check:
 	./bzr selftest $(tests)
+	@echo "Running all tests with no locale."
+	LC_CTYPE= LANG=C LC_ALL= ./bzr selftest $(tests)
 
 check-msgeditor:
 	./bzr --no-plugins selftest -v msgeditor

=== modified file 'NEWS'
--- NEWS	2005-12-13 05:40:51 +0000
+++ NEWS	2005-12-16 15:03:28 +0000
@@ -27,14 +27,21 @@
       This gives better integration with user settings such as ProxyCommand.
       (James Henstridge)
 
-    * Sftp paths can now be relative, or local, according to the IETF draft spec.
-      Paths now take the form:
+    * Sftp paths can now be relative, or local, according to the IETF 
+      draft spec.  Paths now take the form:
       sftp://user:pass@host:port/relative/path
       or
       sftp://user:pass@host:port/%2Fabsolute/path
       or
       sftp://user:pass@host:port//absolute/path
 
+    * Permissions on files underneath .bzr/ are inherited from the .bzr 
+      directory. So for a shared repository, simply doing 'chmod -R g+w .bzr/'
+      will mean that future file will be created with group write permissions.
+
+    * configure.in and config.guess are no longer in the builtin default 
+      ignore list.
+
   IMPROVEMENTS:
 
     * "bzr INIT dir" now initializes the specified directory, and creates 
@@ -92,7 +99,11 @@
     * Faster commits by reading only the headers of affected weave files. 
       (Denys Duchier)
 
-    * New --dry-run option for add.  (Michael Ellerman)
+    * 'bzr add' now takes a --dry-run parameter which shows you what would be
+      added, but doesn't actually add anything. (Michael Ellerman)
+
+    * bzr add now lists how many files were ignored per glob.  add --verbose
+      lists the specific files.  (Aaron Bentley)
 
   BUG FIXES:
 
@@ -168,8 +179,16 @@
     * Much better error message if one of the test suites can't be 
       imported.  (Martin Pool)
 
+    * Make check now runs the test suite twice - once with the default locale,
+      and once with all locales forced to C, to expose bugs. This is not 
+      trivially done within python, so for now its only triggered by running
+      Make check. Integrators and packagers who wish to check for full 
+      platform support should run 'make check' to test the source.
+      (Robert Collins)
+
     * Tests can now run TestSkipped if they can't execute for any reason.
-      (Martin Pool)
+      (Martin Pool) (NB: TestSkipped should only be raised for correctable
+      reasons - see the wiki spec ImprovingBzrTestSuite).
 
   INTERNALS:
 
@@ -227,6 +246,13 @@
  
     * New options to read only the table-of-contents of a weave.  
       (Denys Duchier)
+
+    * Raise NoSuchFile when someone tries to add a non-existant file.
+      (Michael Ellerman)
+
+    * Simplify handling of DivergedBranches in cmd_pull().
+      (Michael Ellerman)
+		   
    
 bzr 0.6 2005-10-28
 

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-11-30 14:55:03 +0000
+++ bzrlib/__init__.py	2005-12-16 01:47:35 +0000
@@ -63,13 +63,11 @@
                   '_darcs',
                   'aclocal.m4',
                   'autom4te*',
-                  'config.guess',
                   'config.h',
                   'config.h.in',
                   'config.log',
                   'config.status',
                   'config.sub',
-                  'configure.in',
                   'stamp-h',
                   'stamp-h.in',
                   'stamp-h1',

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-12-16 03:43:14 +0000
+++ bzrlib/add.py	2005-12-18 04:08:46 +0000
@@ -93,20 +93,26 @@
 
     Returns the number of files added.
     """
-    import os
+    import os, errno
     from bzrlib.errors import BadFileKindError, ForbiddenFileError
     assert isinstance(recurse, bool)
 
     file_list = _prepare_file_list(file_list)
     user_list = file_list[:]
     inv = tree.read_working_inventory()
-    count = 0
+    added = []
+    ignored = {}
 
     for f in file_list:
         rf = tree.relpath(f)
         af = tree.abspath(rf)
 
-        kind = bzrlib.osutils.file_kind(af)
+        try:
+            kind = bzrlib.osutils.file_kind(af)
+        except OSError, e:
+            if hasattr(e, 'errno') and e.errno == errno.ENOENT:
+                raise errors.NoSuchFile(rf)
+            raise
 
         if not InventoryEntry.versionable_kind(kind):
             if f in user_list:
@@ -141,26 +147,31 @@
         elif sub_tree:
             mutter("%r is a bzr tree", f)
         else:
-            count += __add_one(tree, inv, rf, kind, action)
+            added.extend(__add_one(tree, inv, rf, kind, action))
 
         if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
                 subp = bzrlib.osutils.pathjoin(rf, subf)
                 if subf == bzrlib.BZRDIR:
                     mutter("skip control directory %r", subp)
-                elif tree.is_ignored(subp):
-                    mutter("skip ignored sub-file %r", subp)
                 else:
-                    mutter("queue to add sub-file %r", subp)
-                    file_list.append(tree.abspath(subp))
-
-
-    mutter('added %d entries', count)
+                    ignore_glob = tree.is_ignored(subp)
+                    if ignore_glob is not None:
+                        mutter("skip ignored sub-file %r", subp)
+                        if ignore_glob not in ignored:
+                            ignored[ignore_glob] = []
+                        ignored[ignore_glob].append(subp)
+                    else:
+                        mutter("queue to add sub-file %r", subp)
+                        file_list.append(tree.abspath(subp))
+
+
+    mutter('added %d entries', len(added))
     
-    if count > 0:
+    if len(added) > 0:
         tree._write_inventory(inv)
 
-    return count
+    return added, ignored
 
 def __add_one(tree, inv, path, kind, action):
     """Add a file or directory, automatically add unversioned parents."""
@@ -169,10 +180,10 @@
     # This is safe from infinite recursion because the tree root is
     # always versioned.
     if inv.path2id(path) != None:
-        return 0
+        return []
 
     # add parent
-    count = __add_one(tree, inv, dirname(path), 'directory', action)
+    added = __add_one(tree, inv, dirname(path), 'directory', action)
     action(inv, path, kind)
 
-    return count + 1
+    return added + [path]

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2005-12-16 04:42:07 +0000
+++ bzrlib/atomicfile.py	2005-12-18 04:08:46 +0000
@@ -31,7 +31,7 @@
     An encoding can be specified; otherwise the default is ascii.
     """
 
-    def __init__(self, filename, mode='wb', encoding=None):
+    def __init__(self, filename, mode='wb', encoding=None, new_mode=None):
         if mode != 'wb' and mode != 'wt':
             raise ValueError("invalid AtomicFile mode %r" % mode)
 
@@ -48,6 +48,7 @@
 
         self.write = self.f.write
         self.closed = False
+        self._new_mode = new_mode
 
 
     def __repr__(self):
@@ -67,11 +68,14 @@
         self.f = None
         
         try:
-            stat = os.lstat(self.realfilename)
-            os.chmod(self.tmpfilename, stat.st_mode)
+            if self._new_mode is None:
+                self._new_mode = os.lstat(self.realfilename).st_mode
         except OSError, e:
             if e.errno != errno.ENOENT:
                 raise
+        else:
+            os.chmod(self.tmpfilename, self._new_mode)
+
         rename(self.tmpfilename, self.realfilename)
 
 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-12-16 03:43:14 +0000
+++ bzrlib/branch.py	2005-12-18 04:08:46 +0000
@@ -355,7 +355,7 @@
         >>> commit(br1, "lala!", rev_id="REVISION-ID-2B")
         >>> br1.missing_revisions(br2)
         Traceback (most recent call last):
-        DivergedBranches: These branches have diverged.
+        DivergedBranches: These branches have diverged.  Try merge.
         """
         self_history = self.revision_history()
         self_len = len(self_history)
@@ -512,6 +512,10 @@
     _lock_count = None
     _lock = None
     _inventory_weave = None
+    # If set to False (by a plugin, etc) BzrBranch will not set the
+    # mode on created files or directories
+    _set_file_mode = True
+    _set_dir_mode = True
     
     # Map some sort of prefix into a namespace
     # stuff like "revno:10", "revid:", etc.
@@ -562,25 +566,23 @@
         if init:
             self._make_control()
         self._check_format(relax_version_check)
+        self._find_modes()
 
         def get_store(name, compressed=True, prefixed=False):
-            # FIXME: This approach of assuming stores are all entirely compressed
-            # or entirely uncompressed is tidy, but breaks upgrade from 
-            # some existing branches where there's a mixture; we probably 
-            # still want the option to look for both.
             relpath = self._rel_controlfilename(unicode(name))
             store = TextStore(self._transport.clone(relpath),
+                              dir_mode=self._dir_mode,
+                              file_mode=self._file_mode,
                               prefixed=prefixed,
                               compressed=compressed)
-            #if self._transport.should_cache():
-            #    cache_path = pathjoin(self.cache_root, name)
-            #    os.mkdir(cache_path)
-            #    store = bzrlib.store.CachedStore(store, cache_path)
             return store
 
         def get_weave(name, prefixed=False):
             relpath = self._rel_controlfilename(unicode(name))
-            ws = WeaveStore(self._transport.clone(relpath), prefixed=prefixed)
+            ws = WeaveStore(self._transport.clone(relpath),
+                            prefixed=prefixed,
+                            dir_mode=self._dir_mode,
+                            file_mode=self._file_mode)
             if self._transport.should_cache():
                 ws.enable_cache = True
             return ws
@@ -753,7 +755,25 @@
                     f = codecs.getwriter('utf-8')(f, errors='replace')
             path = self._rel_controlfilename(path)
             ctrl_files.append((path, f))
-        self._transport.put_multi(ctrl_files)
+        self._transport.put_multi(ctrl_files, mode=self._file_mode)
+
+    def _find_modes(self, path=None):
+        """Determine the appropriate modes for files and directories."""
+        try:
+            if path is None:
+                path = self._rel_controlfilename('')
+            st = self._transport.stat(path)
+        except errors.TransportNotPossible:
+            self._dir_mode = 0755
+            self._file_mode = 0644
+        else:
+            self._dir_mode = st.st_mode & 07777
+            # Remove the sticky and execute bits for files
+            self._file_mode = self._dir_mode & ~07111
+        if not self._set_dir_mode:
+            self._dir_mode = None
+        if not self._set_file_mode:
+            self._file_mode = None
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
@@ -771,7 +791,12 @@
         bzrlib.weavefile.write_weave_v5(Weave(), sio)
         empty_weave = sio.getvalue()
 
-        dirs = [[], 'revision-store', 'weaves']
+        cfn = self._rel_controlfilename
+        # Since we don't have a .bzr directory, inherit the
+        # mode from the root directory
+        self._find_modes(u'.')
+
+        dirs = ['', 'revision-store', 'weaves']
         files = [('README', 
             "This is a Bazaar-NG control directory.\n"
             "Do not change any files in this directory.\n"),
@@ -784,8 +809,7 @@
             ('inventory.weave', empty_weave),
             ('ancestry.weave', empty_weave)
         ]
-        cfn = self._rel_controlfilename
-        self._transport.mkdir_multi([cfn(d) for d in dirs])
+        self._transport.mkdir_multi([cfn(d) for d in dirs], mode=self._dir_mode)
         self.put_controlfiles(files)
         mutter('created control directory in ' + self._transport.base)
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-12-16 03:43:14 +0000
+++ bzrlib/builtins.py	2005-12-18 04:08:46 +0000
@@ -222,9 +222,9 @@
     add them.
     """
     takes_args = ['file*']
-    takes_options = ['no-recurse', 'dry-run']
+    takes_options = ['no-recurse', 'dry-run', 'verbose']
 
-    def run(self, file_list, no_recurse=False, dry_run=False):
+    def run(self, file_list, no_recurse=False, dry_run=False, verbose=False):
         import bzrlib.add
 
         if dry_run:
@@ -238,7 +238,19 @@
         else:
             action = bzrlib.add.add_action_add_and_print
 
-        bzrlib.add.smart_add(file_list, not no_recurse, action)
+        added, ignored = bzrlib.add.smart_add(file_list, not no_recurse, 
+                                              action)
+        if len(ignored) > 0:
+            for glob in sorted(ignored.keys()):
+                match_len = len(ignored[glob])
+                if verbose:
+                    for path in ignored[glob]:
+                        print "ignored %s matching \"%s\"" % (path, glob)
+                else:
+                    print "ignored %d file(s) matching \"%s\"" % (match_len,
+                                                              glob)
+            print "If you wish to add some of these files, please add them"\
+                " by name."
 
 
 class cmd_mkdir(Command):
@@ -399,16 +411,13 @@
             else:
                 print "Using saved location: %s" % stored_loc
                 location = stored_loc
+
         br_from = Branch.open(location)
         br_to = tree_to.branch
-        try:
-            old_rh = br_to.revision_history()
-            count = tree_to.pull(br_from, overwrite)
-        except DivergedBranches:
-            # FIXME: Just make DivergedBranches display the right message
-            # itself.
-            raise BzrCommandError("These branches have diverged."
-                                  "  Try merge.")
+
+        old_rh = br_to.revision_history()
+        count = tree_to.pull(br_from, overwrite)
+
         if br_to.get_parent() is None or remember:
             br_to.set_parent(location)
         note('%d revision(s) pulled.' % (count,))
@@ -1775,7 +1784,7 @@
             show_ids=False, verbose=False):
         from bzrlib.missing import find_unmerged, iter_log_data
         from bzrlib.log import log_formatter
-        local_branch = bzrlib.branch.Branch.open_containing(".")[0]
+        local_branch = bzrlib.branch.Branch.open_containing(u".")[0]
         parent = local_branch.get_parent()
         if other_branch is None:
             other_branch = parent

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-12-16 03:43:14 +0000
+++ bzrlib/errors.py	2005-12-18 04:08:46 +0000
@@ -277,7 +277,7 @@
 
 class DivergedBranches(BzrError):
     def __init__(self, branch1, branch2):
-        BzrError.__init__(self, "These branches have diverged.")
+        BzrError.__init__(self, "These branches have diverged.  Try merge.")
         self.branch1 = branch1
         self.branch2 = branch2
 

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2005-11-30 00:25:50 +0000
+++ bzrlib/store/__init__.py	2005-12-15 21:39:34 +0000
@@ -220,7 +220,8 @@
                 pass
         raise KeyError(fileid)
 
-    def __init__(self, a_transport, prefixed=False, compressed=False):
+    def __init__(self, a_transport, prefixed=False, compressed=False,
+                 dir_mode=None, file_mode=None):
         assert isinstance(a_transport, transport.Transport)
         super(TransportStore, self).__init__()
         self._transport = a_transport
@@ -228,6 +229,11 @@
         self._compressed = compressed
         self._suffixes = set()
 
+        # It is okay for these to be None, it just means they
+        # will just use the filesystem defaults
+        self._dir_mode = dir_mode
+        self._file_mode = file_mode
+
     def _iter_files_recursive(self):
         """Iterate through the files in the transport."""
         for quoted_relpath in self._transport.iter_files_recursive():
@@ -303,37 +309,6 @@
     return bzrlib.store.text.TextStore(transport.memory.MemoryTransport())
         
 
-class CachedStore(Store):
-    """A store that caches data locally, to avoid repeated downloads.
-    The precacache method should be used to avoid server round-trips for
-    every piece of data.
-    """
-
-    def __init__(self, store, cache_dir):
-        super(CachedStore, self).__init__()
-        self.source_store = store
-        # This clones the source store type with a locally bound
-        # transport. FIXME: it assumes a constructor is == cloning.
-        # clonable store - it might be nicer to actually have a clone()
-        # or something. RBC 20051003
-        self.cache_store = store.__class__(LocalTransport(cache_dir))
-
-    def get(self, id):
-        mutter("Cache add %s", id)
-        if id not in self.cache_store:
-            self.cache_store.add(self.source_store.get(id), id)
-        return self.cache_store.get(id)
-
-    def has_id(self, fileid, suffix=None):
-        """See Store.has_id."""
-        if self.cache_store.has_id(fileid, suffix):
-            return True
-        if self.source_store.has_id(fileid, suffix):
-            # We could asynchronously copy at this time
-            return True
-        return False
-
-
 def copy_all(store_from, store_to):
     """Copy all ids from one store to another."""
     # TODO: Optional progress indicator

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2005-11-10 02:11:13 +0000
+++ bzrlib/store/text.py	2005-12-15 21:11:09 +0000
@@ -65,15 +65,15 @@
 
     def _try_put(self, fn, f):
         try:
-            self._transport.put(fn, f)
+            self._transport.put(fn, f, mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
             try:
-                self._transport.mkdir(os.path.dirname(fn))
+                self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
             except FileExists:
                 pass
-            self._transport.put(fn, f)
+            self._transport.put(fn, f, mode=self._file_mode)
 
     def _get(self, fn):
         if fn.endswith('.gz'):
@@ -95,15 +95,17 @@
             raise KeyError(fileid + '-' + str(suffix))
 
         try:
-            result = other._transport.copy_to([path], self._transport, pb=pb)
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
             try:
-                self._transport.mkdir(hash_prefix(fileid)[:-1])
+                self._transport.mkdir(hash_prefix(fileid)[:-1], mode=self._dir_mode)
             except FileExists:
                 pass
-            result = other._transport.copy_to([path], self._transport, pb=pb)
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
 
         if result != 1:
             raise BzrError('Unable to copy file: %r' % (path,))

=== modified file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	2005-12-08 07:35:29 +0000
+++ bzrlib/store/weave.py	2005-12-15 21:11:09 +0000
@@ -40,9 +40,11 @@
     """
     FILE_SUFFIX = '.weave'
 
-    def __init__(self, transport, prefixed=False, precious=False):
-        self._transport = transport
-        self._prefixed = prefixed
+    def __init__(self, transport, prefixed=False, precious=False,
+                 dir_mode=None, file_mode=None):
+        super(WeaveStore, self).__init__(transport,
+                dir_mode=dir_mode, file_mode=file_mode,
+                prefixed=prefixed, compressed=False)
         self._precious = precious
 
     def filename(self, file_id):
@@ -67,10 +69,10 @@
     def _put(self, file_id, f):
         if self._prefixed:
             try:
-                self._transport.mkdir(hash_prefix(file_id))
+                self._transport.mkdir(hash_prefix(file_id), mode=self._dir_mode)
             except FileExists:
                 pass
-        return self._transport.put(self.filename(file_id), f)
+        return self._transport.put(self.filename(file_id), f, mode=self._file_mode)
 
     def get_weave(self, file_id, transaction):
         weave = transaction.map.find_weave(file_id)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2005-12-16 03:43:14 +0000
+++ bzrlib/tests/__init__.py	2005-12-18 04:08:46 +0000
@@ -273,6 +273,16 @@
             raise AssertionError('pattern "%s" not found in "%s"'
                     % (needle_re, haystack))
 
+    def AssertSubset(self, sublist, superlist):
+        """Assert that every entry in sublist is present in superlist."""
+        missing = []
+        for entry in sublist:
+            if entry not in superlist:
+                missing.append(entry)
+        if len(missing) > 0:
+            raise AssertionError("value(s) %r not present in container %r" % 
+                                 (missing, superlist))
+
     def _startLogFile(self):
         """Send bzr and test log messages to a temporary file.
 
@@ -636,63 +646,64 @@
     global MODULES_TO_DOCTEST
 
     testmod_names = [ \
-                    'bzrlib.tests.test_ancestry',
-                    'bzrlib.tests.test_annotate',
-                    'bzrlib.tests.test_api',
-                    'bzrlib.tests.test_bad_files',
-                    'bzrlib.tests.test_branch',
-                    'bzrlib.tests.test_command',
-                    'bzrlib.tests.test_commit',
-                    'bzrlib.tests.test_commit_merge',
-                    'bzrlib.tests.test_config',
-                    'bzrlib.tests.test_conflicts',
-                    'bzrlib.tests.test_diff',
-                    'bzrlib.tests.test_fetch',
-                    'bzrlib.tests.test_gpg',
-                    'bzrlib.tests.test_graph',
-                    'bzrlib.tests.test_hashcache',
-                    'bzrlib.tests.test_http',
-                    'bzrlib.tests.test_identitymap',
-                    'bzrlib.tests.test_inv',
-                    'bzrlib.tests.test_log',
-                    'bzrlib.tests.test_merge',
-                    'bzrlib.tests.test_merge3',
-                    'bzrlib.tests.test_merge_core',
-                    'bzrlib.tests.test_missing',
-                    'bzrlib.tests.test_msgeditor',
-                    'bzrlib.tests.test_nonascii',
-                    'bzrlib.tests.test_options',
-                    'bzrlib.tests.test_osutils',
-                    'bzrlib.tests.test_parent',
-                    'bzrlib.tests.test_plugins',
-                    'bzrlib.tests.test_remove',
-                    'bzrlib.tests.test_revision',
-                    'bzrlib.tests.test_revision_info',
-                    'bzrlib.tests.test_revisionnamespaces',
-                    'bzrlib.tests.test_revprops',
-                    'bzrlib.tests.test_reweave',
-                    'bzrlib.tests.test_rio',
-                    'bzrlib.tests.test_sampler',
-                    'bzrlib.tests.test_selftest',
-                    'bzrlib.tests.test_setup',
-                    'bzrlib.tests.test_sftp_transport',
-                    'bzrlib.tests.test_smart_add',
-                    'bzrlib.tests.test_source',
-                    'bzrlib.tests.test_status',
-                    'bzrlib.tests.test_store',
-                    'bzrlib.tests.test_testament',
-                    'bzrlib.tests.test_trace',
-                    'bzrlib.tests.test_transactions',
-                    'bzrlib.tests.test_transport',
-                    'bzrlib.tests.test_tsort',
-                    'bzrlib.tests.test_ui',
-                    'bzrlib.tests.test_uncommit',
-                    'bzrlib.tests.test_upgrade',
-                    'bzrlib.tests.test_weave',
-                    'bzrlib.tests.test_whitebox',
-                    'bzrlib.tests.test_workingtree',
-                    'bzrlib.tests.test_xml',
-                    ]
+                   'bzrlib.tests.test_ancestry',
+                   'bzrlib.tests.test_annotate',
+                   'bzrlib.tests.test_api',
+                   'bzrlib.tests.test_bad_files',
+                   'bzrlib.tests.test_branch',
+                   'bzrlib.tests.test_command',
+                   'bzrlib.tests.test_commit',
+                   'bzrlib.tests.test_commit_merge',
+                   'bzrlib.tests.test_config',
+                   'bzrlib.tests.test_conflicts',
+                   'bzrlib.tests.test_diff',
+                   'bzrlib.tests.test_fetch',
+                   'bzrlib.tests.test_gpg',
+                   'bzrlib.tests.test_graph',
+                   'bzrlib.tests.test_hashcache',
+                   'bzrlib.tests.test_http',
+                   'bzrlib.tests.test_identitymap',
+                   'bzrlib.tests.test_inv',
+                   'bzrlib.tests.test_log',
+                   'bzrlib.tests.test_merge',
+                   'bzrlib.tests.test_merge3',
+                   'bzrlib.tests.test_merge_core',
+                   'bzrlib.tests.test_missing',
+                   'bzrlib.tests.test_msgeditor',
+                   'bzrlib.tests.test_nonascii',
+                   'bzrlib.tests.test_options',
+                   'bzrlib.tests.test_osutils',
+                   'bzrlib.tests.test_parent',
+                   'bzrlib.tests.test_permissions',
+                   'bzrlib.tests.test_plugins',
+                   'bzrlib.tests.test_remove',
+                   'bzrlib.tests.test_revision',
+                   'bzrlib.tests.test_revision_info',
+                   'bzrlib.tests.test_revisionnamespaces',
+                   'bzrlib.tests.test_revprops',
+                   'bzrlib.tests.test_reweave',
+                   'bzrlib.tests.test_rio',
+                   'bzrlib.tests.test_sampler',
+                   'bzrlib.tests.test_selftest',
+                   'bzrlib.tests.test_setup',
+                   'bzrlib.tests.test_sftp_transport',
+                   'bzrlib.tests.test_smart_add',
+                   'bzrlib.tests.test_source',
+                   'bzrlib.tests.test_status',
+                   'bzrlib.tests.test_store',
+                   'bzrlib.tests.test_testament',
+                   'bzrlib.tests.test_trace',
+                   'bzrlib.tests.test_transactions',
+                   'bzrlib.tests.test_transport',
+                   'bzrlib.tests.test_tsort',
+                   'bzrlib.tests.test_ui',
+                   'bzrlib.tests.test_uncommit',
+                   'bzrlib.tests.test_upgrade',
+                   'bzrlib.tests.test_weave',
+                   'bzrlib.tests.test_whitebox',
+                   'bzrlib.tests.test_workingtree',
+                   'bzrlib.tests.test_xml',
+                   ]
 
     TestCase.BZRPATH = osutils.pathjoin(
             osutils.realpath(osutils.dirname(bzrlib.__path__[0])), 'bzr')

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2005-12-01 23:52:16 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2005-12-18 04:08:46 +0000
@@ -15,6 +15,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# Mr. Smoketoomuch: I'm sorry?
+# Mr. Bounder: You'd better cut down a little then.
+# Mr. Smoketoomuch: Oh, I see! Smoke too much so I'd better cut down a little
+#                   then!
 
 """Black-box tests for bzr.
 
@@ -666,13 +670,22 @@
     def test_add_reports(self):
         """add command prints the names of added files."""
         b = Branch.initialize('.')
-        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt', 'CVS'])
         out = self.run_bzr_captured(['add'], retcode=0)[0]
         # the ordering is not defined at the moment
         results = sorted(out.rstrip('\n').split('\n'))
-        self.assertEquals(['added dir',
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'added dir',
                            'added dir/sub.txt',
-                           'added top.txt',],
+                           'added top.txt',
+                           'ignored 1 file(s) matching "CVS"'],
+                          results)
+        out = self.run_bzr_captured(['add', '-v'], retcode=0)[0]
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'ignored CVS matching "CVS"'],
                           results)
 
     def test_add_quiet_is(self):

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2005-12-02 04:16:07 +0000
+++ bzrlib/tests/stub_sftp.py	2005-12-18 04:08:46 +0000
@@ -23,6 +23,7 @@
 from paramiko import ServerInterface, SFTPServerInterface, SFTPServer, SFTPAttributes, \
     SFTPHandle, SFTP_OK, AUTH_SUCCESSFUL, OPEN_SUCCEEDED
 from bzrlib.osutils import pathjoin
+from bzrlib.trace import mutter
 
 
 class StubServer (ServerInterface):
@@ -50,6 +51,7 @@
     def chattr(self, attr):
         # python doesn't have equivalents to fchown or fchmod, so we have to
         # use the stored filename
+        mutter('Changing permissions on %s to %s', self.filename, attr)
         try:
             SFTPServer.set_file_attr(self.filename, attr)
         except OSError, e:
@@ -64,6 +66,13 @@
     def _realpath(self, path):
         return self.root + self.canonicalize(path)
 
+    def chattr(self, path, attr):
+        try:
+            SFTPServer.set_file_attr(path, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
     def list_folder(self, path):
         path = self._realpath(path)
         try:
@@ -96,10 +105,14 @@
         try:
             if hasattr(os, 'O_BINARY'):
                 flags |= os.O_BINARY
-            fd = os.open(path, flags)
+            if (attr is not None) and hasattr(attr, 'st_mode'):
+                fd = os.open(path, flags, attr.st_mode)
+            else:
+                fd = os.open(path, flags)
         except OSError, e:
             return SFTPServer.convert_errno(e.errno)
         if (flags & os.O_CREAT) and (attr is not None):
+            attr._flags &= ~attr.FLAG_PERMISSIONS
             SFTPServer.set_file_attr(path, attr)
         if flags & os.O_WRONLY:
             fstr = 'wb'
@@ -138,8 +151,12 @@
     def mkdir(self, path, attr):
         path = self._realpath(path)
         try:
-            os.mkdir(path)
+            if attr is not None and hasattr(attr, 'st_mode'):
+                os.mkdir(path, attr.st_mode)
+            else:
+                os.mkdir(path)
             if attr is not None:
+                attr._flags &= ~attr.FLAG_PERMISSIONS
                 SFTPServer.set_file_attr(path, attr)
         except OSError, e:
             return SFTPServer.convert_errno(e.errno)

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2005-12-13 05:48:04 +0000
+++ bzrlib/tests/test_msgeditor.py	2005-12-14 09:01:40 +0000
@@ -23,28 +23,25 @@
 from bzrlib.msgeditor import make_commit_message_template
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 
-from bzrlib.tests.treeshape import build_tree_contents
-
-
-def make_uncommitted_tree():
-    """Build a branch with uncommitted changes in the cwd."""
-    b = Branch.initialize('.')
-    working_tree = b.working_tree()
-    filename = u'hell\u00d8'
-    try:
-        build_tree_contents([(filename, 'contents of hello')])
-    except UnicodeEncodeError:
-        raise TestSkipped("can't build unicode working tree in "
+
+class MsgEditorTest(TestCaseInTempDir):
+
+    def make_uncommitted_tree(self):
+        """Build a branch with uncommitted unicode named changes in the cwd."""
+        b = Branch.initialize('.')
+        working_tree = b.working_tree()
+        filename = u'hell\u00d8'
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
                 "filesystem encoding %s" % sys.getfilesystemencoding())
-    working_tree.add(filename)
-    return working_tree
-
-
-class MsgEditorTest(TestCaseInTempDir):
-
+        working_tree.add(filename)
+        return working_tree
+    
     def test_commit_template(self):
         """Test building a commit message template"""
-        working_tree = make_uncommitted_tree()
+        working_tree = self.make_uncommitted_tree()
         template = make_commit_message_template(working_tree, None)
         self.assertEqualDiff(template,
 u"""\

=== added file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_permissions.py	2005-12-18 04:08:46 +0000
@@ -0,0 +1,321 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for bzr setting permissions.
+
+Files which are created underneath .bzr/ should inherit its permissions.
+So if the directory is group writable, the files and subdirs should be as well.
+
+In the future, when we have Repository/Branch/Checkout information, the
+permissions should be inherited individually, rather than all be the same.
+
+TODO: jam 20051215 There are no tests for ftp yet, because we have no ftp server
+TODO: jam 20051215 Currently the default behavior for 'bzr branch' is just 
+                   defined by the local umask. This isn't terrible, is it
+                   the truly desired behavior?
+"""
+
+import os
+import sys
+import stat
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.tests.test_transport import check_mode
+
+
+def chmod_r(base, file_mode, dir_mode):
+    """Recursively chmod from a base directory"""
+    assert os.path.isdir(base)
+    os.chmod(base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            os.chmod(p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            os.chmod(p, file_mode)
+
+
+def check_mode_r(test, base, file_mode, dir_mode, include_base=True):
+    """Check that all permissions match
+
+    :param test: The TestCase being run
+    :param base: The path to the root directory to check
+    :param file_mode: The mode for all files
+    :param dir_mode: The mode for all directories
+    :param include_base: If false, only check the subdirectories
+    """
+    assert os.path.isdir(base)
+    if include_base:
+        check_mode(test, base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            check_mode(test, p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            check_mode(test, p, file_mode)
+
+def assertEqualMode(test, mode, mode_test):
+    test.assertEqual(mode, mode_test,
+                     'mode mismatch %o != %o' % (mode, mode_test))
+
+class TestPermissions(TestCaseInTempDir):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        open('a', 'wb').write('foo\n')
+        t.add('a')
+        t.commit('foo')
+
+        # Delete them because we are modifying the filesystem underneath them
+        del b, t 
+        chmod_r('.bzr', 0644, 0755)
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 0755, b._dir_mode)
+        assertEqualMode(self, 0644, b._file_mode)
+
+        # Modifying a file shouldn't break the permissions
+        open('a', 'wb').write('foo2\n')
+        t.commit('foo2')
+        # The mode should be maintained after commit
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        # Adding a new file should maintain the permissions
+        open('b', 'wb').write('new b\n')
+        t.add('b')
+        t.commit('new b')
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        del b, t
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 0775)
+        check_mode_r(self, '.bzr', 0664, 0775)
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 0775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
+
+        open('a', 'wb').write('foo3\n')
+        t.commit('foo3')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        open('c', 'wb').write('new c\n')
+        t.add('c')
+        t.commit('new c')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        # Test the group sticky bit
+        del b, t
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 02775)
+        check_mode_r(self, '.bzr', 0664, 02775)
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 02775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
+
+        open('a', 'wb').write('foo4\n')
+        t.commit('foo4')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+        open('d', 'wb').write('new d\n')
+        t.add('d')
+        t.commit('new d')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+    def test_disable_set_mode(self):
+        # TODO: jam 20051215 Ultimately, this test should probably test that
+        #                    extra chmod calls aren't being made
+        import bzrlib.branch
+        try:
+            b = Branch.initialize(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_dir_mode = False
+            b = Branch.open(u'.')
+            self.assertEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_file_mode = False
+            b = Branch.open(u'.')
+            self.assertEqual(None, b._dir_mode)
+            self.assertEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_dir_mode = True
+            b = Branch.open(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_file_mode = True
+            b = Branch.open(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+        finally:
+            bzrlib.branch.BzrBranch._set_dir_mode = True
+            bzrlib.branch.BzrBranch._set_file_mode = True
+
+    def test_new_branch(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        os.mkdir('a')
+        mode = stat.S_IMODE(os.stat('a').st_mode)
+        b = Branch.initialize('a')
+        assertEqualMode(self, mode, b._dir_mode)
+        assertEqualMode(self, mode & ~07111, b._file_mode)
+
+        os.mkdir('b')
+        os.chmod('b', 02777)
+        b = Branch.initialize('b')
+        assertEqualMode(self, 02777, b._dir_mode)
+        assertEqualMode(self, 00666, b._file_mode)
+        check_mode_r(self, 'b/.bzr', 00666, 02777)
+
+        os.mkdir('c')
+        os.chmod('c', 02750)
+        b = Branch.initialize('c')
+        assertEqualMode(self, 02750, b._dir_mode)
+        assertEqualMode(self, 00640, b._file_mode)
+        check_mode_r(self, 'c/.bzr', 00640, 02750)
+
+        os.mkdir('d')
+        os.chmod('d', 0700)
+        b = Branch.initialize('d')
+        assertEqualMode(self, 0700, b._dir_mode)
+        assertEqualMode(self, 0600, b._file_mode)
+        check_mode_r(self, 'd/.bzr', 00600, 0700)
+
+
+class TestSftpPermissions(TestCaseWithSFTPServer):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        # Though it would be nice to test that SFTP to a server
+        # which does support chmod has the right effect
+
+        from bzrlib.transport.sftp import SFTPTransport
+
+        # We don't actually use it directly, we just want to
+        # keep the connection open, since StubSFTPServer only
+        # allows 1 connection
+        self.delayed_setup()
+        _transport = SFTPTransport(self._sftp_url)
+
+        os.mkdir('local')
+        b_local = Branch.initialize(u'local')
+        t_local = b_local.working_tree()
+        open('local/a', 'wb').write('foo\n')
+        t_local.add('a')
+        t_local.commit('foo')
+
+        # Delete them because we are modifying the filesystem underneath them
+        del b_local, t_local 
+        chmod_r('local/.bzr', 0644, 0755)
+        check_mode_r(self, 'local/.bzr', 0644, 0755)
+
+        b_local = Branch.open(u'local')
+        t_local = b_local.working_tree()
+        assertEqualMode(self, 0755, b_local._dir_mode)
+        assertEqualMode(self, 0644, b_local._file_mode)
+
+        os.mkdir('sftp')
+        # Why does self._sftp_url end with a slash????
+        sftp_url = self._sftp_url + 'sftp'
+        b_sftp = Branch.initialize(sftp_url)
+
+        b_sftp.pull(b_local)
+        del b_sftp
+        chmod_r('sftp/.bzr', 0644, 0755)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0755, b_sftp._dir_mode)
+        assertEqualMode(self, 0644, b_sftp._file_mode)
+
+        open('local/a', 'wb').write('foo2\n')
+        t_local.commit('foo2')
+        b_sftp.pull(b_local)
+        # The mode should be maintained after commit
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        open('local/b', 'wb').write('new b\n')
+        t_local.add('b')
+        t_local.commit('new b')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        del b_sftp
+        # Recursively update the modes of all files
+        chmod_r('sftp/.bzr', 0664, 0775)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0775, b_sftp._dir_mode)
+        assertEqualMode(self, 0664, b_sftp._file_mode)
+
+        open('local/a', 'wb').write('foo3\n')
+        t_local.commit('foo3')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        open('local/c', 'wb').write('new c\n')
+        t_local.add('c')
+        t_local.commit('new c')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+    def test_sftp_server_modes(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        umask = 0022
+        original_umask = os.umask(umask)
+
+        try:
+            from bzrlib.transport.sftp import SFTPTransport
+            self.delayed_setup()
+            t = SFTPTransport(self._sftp_url)
+            # Direct access should be masked by umask
+            t._sftp_open_exclusive('a', mode=0666).write('foo\n')
+            check_mode(self, 'a', 0666 &~umask)
+
+            # but Transport overrides umask
+            t.put('b', 'txt', mode=0666)
+            check_mode(self, 'b', 0666)
+
+            t._sftp.mkdir('c', mode=0777)
+            check_mode(self, 'c', 0777 &~umask)
+
+            t.mkdir('d', mode=0777)
+            check_mode(self, 'd', 0777)
+        finally:
+            os.umask(original_umask)
+
+

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2005-12-13 05:40:51 +0000
+++ bzrlib/tests/test_selftest.py	2005-12-14 09:01:40 +0000
@@ -20,9 +20,13 @@
 import sys
 import unittest
 
-from bzrlib.tests import TestCase, _load_module_by_name, \
-        TestSkipped
-import bzrlib.tests
+from bzrlib.tests import (
+                          _load_module_by_name,
+                          TestCase,
+                          TestCaseInTempDir,
+                          TestSkipped,
+                          TextTestRunner,
+                          )
 
 
 class SelftestTests(TestCase):
@@ -38,6 +42,7 @@
 
 
 class MetaTestLog(TestCase):
+
     def test_logging(self):
         """Test logs are captured when a test fails."""
         self.log('a test message')
@@ -45,13 +50,25 @@
         self.assertContainsRe(self._get_log(), 'a test message\n')
 
 
+class TestTreeShape(TestCaseInTempDir):
+
+    def test_unicode_paths(self):
+        filename = u'hell\u00d8'
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
+                "filesystem encoding %s" % sys.getfilesystemencoding())
+        self.failUnlessExists(filename)
+
+
 class TestSkippedTest(TestCase):
     """Try running a test which is skipped, make sure it's reported properly."""
     def test_skipped_test(self):
         # must be hidden in here so it's not run as a real test
         def skipping_test():
             raise TestSkipped('test intentionally skipped')
-        runner = bzrlib.tests.TextTestRunner(stream=self._log_file)
+        runner = TextTestRunner(stream=self._log_file)
         test = unittest.FunctionTestCase(skipping_test)
         result = runner.run(test)
         self.assertTrue(result.wasSuccessful())

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2005-12-02 00:07:11 +0000
+++ bzrlib/tests/test_sftp_transport.py	2005-12-18 04:08:46 +0000
@@ -18,7 +18,7 @@
 import socket
 import threading
 
-from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
 from bzrlib.tests.test_transport import TestTransportMixIn
 import bzrlib.errors as errors
 from bzrlib.osutils import pathjoin, lexists
@@ -104,6 +104,8 @@
 
     def setUp(self):
         TestCaseInTempDir.setUp(self)
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
         self._root = self.test_dir
         self._is_setup = False
 
@@ -190,6 +192,11 @@
 
 
 class SFTPNonServerTest(TestCase):
+    def setUp(self):
+        TestCase.setUp(self)
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+
     def test_parse_url(self):
         from bzrlib.transport.sftp import SFTPTransport
         s = SFTPTransport('sftp://simple.example.com/%2fhome/source', clone_from=fake)
@@ -306,8 +313,3 @@
         self.assertEquals(b2.revision_history(), ['a1', 'a2'])
 
 
-if not paramiko_loaded:
-    # TODO: Skip these
-    del SFTPTransportTest
-    del SFTPNonServerTest
-    del SFTPBranchTest

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2005-12-09 07:12:46 +0000
+++ bzrlib/tests/test_smart_add.py	2005-12-15 08:32:03 +0000
@@ -3,9 +3,10 @@
 
 from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.branch import Branch
-from bzrlib.errors import NotBranchError
+from bzrlib.errors import NotBranchError, NoSuchFile
 from bzrlib.inventory import InventoryFile, Inventory
 from bzrlib.workingtree import WorkingTree
+from bzrlib.add import smart_add
 
 class TestSmartAdd(TestCaseInTempDir):
 
@@ -85,6 +86,26 @@
         self.capture('add --dry-run .')
         eq(list(t.unknowns()), ['inertiatic'])
 
+    def test_add_non_existant(self):
+        """Test smart-adding a file that does not exist."""
+        from bzrlib.add import smart_add
+        branch = Branch.initialize(u".")
+        self.assertRaises(NoSuchFile, smart_add, 'non-existant-file')
+
+    def test_returns(self):
+        """Correctly returns added/ignored files"""
+        from bzrlib.commands import run_bzr
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        self.build_tree(['inertiatic/', 'inertiatic/esp', 'inertiatic/CVS', 
+                        'inertiatic/foo.pyc'])
+        added, ignored = smart_add(u'.')
+        self.AssertSubset(('inertiatic', 'inertiatic/esp'), added)
+        self.AssertSubset(('CVS', '*.py[oc]'), ignored)
+        self.AssertSubset(('inertiatic/CVS',), ignored['CVS'])
+        self.AssertSubset(('inertiatic/foo.pyc',), ignored['*.py[oc]'])
+
+
 class TestSmartAddBranch(TestCaseInTempDir):
     """Test smart adds with a specified branch."""
 
@@ -157,6 +178,7 @@
         for path in paths:
             self.assertNotEqual(tree.path2id(path), None)
 
+
 class TestAddActions(TestCaseInTempDir):
 
     def test_null(self):

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2005-12-01 21:49:24 +0000
+++ bzrlib/tests/test_store.py	2005-12-18 04:08:46 +0000
@@ -92,7 +92,8 @@
         self.assertEqual(store.total_size(), (2, 55))
         
     def test__relpath_suffixed(self):
-        my_store = TextStore(MockTransport(), True, compressed=True)
+        my_store = TextStore(MockTransport(),
+                             prefixed=True, compressed=True)
         my_store.register_suffix('dsc')
         self.assertEqual('45/foo.dsc', my_store._relpath('foo', ['dsc']))
 
@@ -357,7 +358,8 @@
                          my_store.get('missing', 'sig').read())
 
     def test___iter__no_suffix(self):
-        my_store = TextStore(MemoryTransport(), False, compressed=False)
+        my_store = TextStore(MemoryTransport(),
+                             prefixed=False, compressed=False)
         stream = StringIO("content")
         my_store.add(stream, "foo")
         self.assertEqual(set(['foo']),
@@ -382,7 +384,8 @@
 
     def test_copy_suffixes(self):
         from_store = self.get_populated_store()
-        to_store = TextStore(MemoryTransport(), True, compressed=True)
+        to_store = TextStore(MemoryTransport(),
+                             prefixed=True, compressed=True)
         to_store.register_suffix('sig')
         copy_all(from_store, to_store)
         self.assertEqual(1, len(to_store))

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2005-12-16 03:43:14 +0000
+++ bzrlib/tests/test_transport.py	2005-12-18 04:08:46 +0000
@@ -16,6 +16,8 @@
 
 
 import os
+import sys
+import stat
 from cStringIO import StringIO
 
 from bzrlib.errors import (NoSuchFile, FileExists,
@@ -34,6 +36,21 @@
     f.close()
     del f
 
+
+if sys.platform != 'win32':
+    def check_mode(test, path, mode):
+        """Check that a particular path has the correct mode."""
+        actual_mode = stat.S_IMODE(os.stat(path).st_mode)
+        test.assertEqual(mode, actual_mode,
+            'mode of %r incorrect (%o != %o)' % (path, mode, actual_mode))
+else:
+    def check_mode(test, path, mode):
+        """On win32 chmod doesn't have any effect, 
+        so don't actually check anything
+        """
+        return
+
+
 class TestTransport(TestCase):
     """Test the non transport-concrete class functionality."""
 
@@ -109,6 +126,14 @@
     def test_put(self):
         t = self.get_transport()
 
+        # TODO: jam 20051215 No need to do anything if the test is readonly
+        #                    origininally it was thought that it would give
+        #                    more of a workout to readonly tests. By now the
+        #                    suite is probably thorough enough without testing
+        #                    readonly protocols in write sections
+        #                    The only thing that needs to be tested is that the
+        #                    right error is raised
+
         if self.readonly:
             self.assertRaises(TransportNotPossible,
                     t.put, 'a', 'some text for a\n')
@@ -159,6 +184,26 @@
             self.assertRaises(NoSuchFile,
                     t.put, 'path/doesnt/exist/c', 'contents')
 
+        if not self.readonly:
+            t.put('mode644', 'test text\n', mode=0644)
+            check_mode(self, 'mode644', 0644)
+
+            t.put('mode666', 'test text\n', mode=0666)
+            check_mode(self, 'mode666', 0666)
+
+            t.put('mode600', 'test text\n', mode=0600)
+            check_mode(self, 'mode600', 0600)
+
+            # Yes, you can put a file such that it becomes readonly
+            t.put('mode400', 'test text\n', mode=0400)
+            check_mode(self, 'mode400', 0400)
+
+            t.put_multi([('mmode644', 'text\n')], mode=0644)
+            check_mode(self, 'mmode644', 0644)
+
+        # TODO: jam 20051215 test put_multi with a mode. I didn't bother because
+        #                    it seems most people don't like the _multi functions
+
     def test_put_file(self):
         t = self.get_transport()
 
@@ -213,6 +258,24 @@
         self.check_file_contents('f5', 'here is some text\nand a bit more\n')
         self.check_file_contents('f6', 'some text for the\nthird file created\n')
 
+        if not self.readonly:
+            sio = StringIO('test text\n')
+            t.put('mode644', sio, mode=0644)
+            check_mode(self, 'mode644', 0644)
+
+            a = open('mode644', 'rb')
+            t.put('mode666', a, mode=0666)
+            check_mode(self, 'mode666', 0666)
+
+            a = open('mode644', 'rb')
+            t.put('mode600', a, mode=0600)
+            check_mode(self, 'mode600', 0600)
+
+            # Yes, you can put a file such that it becomes readonly
+            a = open('mode644', 'rb')
+            t.put('mode400', a, mode=0400)
+            check_mode(self, 'mode400', 0400)
+
     def test_mkdir(self):
         t = self.get_transport()
 
@@ -281,6 +344,25 @@
         for f in ('dir_a/a', 'dir_b/b'):
             self.assertEqual(t.get(f).read(), open(f, 'rb').read())
 
+        if not self.readonly:
+            # Test mkdir with a mode
+            t.mkdir('dmode755', mode=0755)
+            check_mode(self, 'dmode755', 0755)
+
+            t.mkdir('dmode555', mode=0555)
+            check_mode(self, 'dmode555', 0555)
+
+            t.mkdir('dmode777', mode=0777)
+            check_mode(self, 'dmode777', 0777)
+
+            t.mkdir('dmode700', mode=0700)
+            check_mode(self, 'dmode700', 0700)
+
+            # TODO: jam 20051215 test mkdir_multi with a mode
+            t.mkdir_multi(['mdmode755'], mode=0755)
+            check_mode(self, 'mdmode755', 0755)
+
+
     def test_copy_to(self):
         import tempfile
         from bzrlib.transport.local import LocalTransport
@@ -290,9 +372,11 @@
         files = ['a', 'b', 'c', 'd']
         self.build_tree(files)
 
-        dtmp = tempfile.mkdtemp(dir=u'.', prefix='test-transport-')
-        dtmp_base = os.path.basename(dtmp)
-        local_t = LocalTransport(dtmp)
+        def get_temp_local():
+            dtmp = tempfile.mkdtemp(dir=u'.', prefix='test-transport-')
+            dtmp_base = os.path.basename(dtmp)
+            return dtmp_base, LocalTransport(dtmp)
+        dtmp_base, local_t = get_temp_local()
 
         t.copy_to(files, local_t)
         for f in files:
@@ -308,11 +392,9 @@
         os.mkdir(pathjoin(dtmp_base, 'e'))
         t.copy_to(['e/f'], local_t)
 
-        del dtmp, dtmp_base, local_t
+        del dtmp_base, local_t
 
-        dtmp = tempfile.mkdtemp(dir=u'.', prefix='test-transport-')
-        dtmp_base = os.path.basename(dtmp)
-        local_t = LocalTransport(dtmp)
+        dtmp_base, local_t = get_temp_local()
 
         files = ['a', 'b', 'c', 'd']
         t.copy_to(iter(files), local_t)
@@ -320,7 +402,13 @@
             self.assertEquals(open(f, 'rb').read(),
                     open(pathjoin(dtmp_base, f), 'rb').read())
 
-        del dtmp, dtmp_base, local_t
+        del dtmp_base, local_t
+
+        for mode in (0666, 0644, 0600, 0400):
+            dtmp_base, local_t = get_temp_local()
+            t.copy_to(files, local_t, mode=mode)
+            for f in files:
+                check_mode(self, os.path.join(dtmp_base, f), mode)
 
     def test_append(self):
         t = self.get_transport()

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-12-16 05:13:21 +0000
+++ bzrlib/transport/__init__.py	2005-12-18 04:08:46 +0000
@@ -221,30 +221,37 @@
             yield self.get(relpath)
             count += 1
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param f:       File-like or string object.
+        :param mode: The mode for the newly created file, 
+                     None means just use the default
         """
         raise NotImplementedError
 
-    def put_multi(self, files, pb=None):
+    def put_multi(self, files, mode=None, pb=None):
         """Put a set of files or strings into the location.
 
         :param files: A list of tuples of relpath, file object [(path1, file1), (path2, file2),...]
         :param pb:  An optional ProgressBar for indicating percent done.
+        :param mode: The mode for the newly created files
         :return: The number of files copied.
         """
-        return self._iterate_over(files, self.put, pb, 'put', expand=True)
+        def put(path, f):
+            self.put(path, f, mode=mode)
+        return self._iterate_over(files, put, pb, 'put', expand=True)
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         raise NotImplementedError
 
-    def mkdir_multi(self, relpaths, pb=None):
+    def mkdir_multi(self, relpaths, mode=None, pb=None):
         """Create a group of directories"""
-        return self._iterate_over(relpaths, self.mkdir, pb, 'mkdir', expand=False)
+        def mkdir(path):
+            self.mkdir(path, mode=mode)
+        return self._iterate_over(relpaths, mkdir, pb, 'mkdir', expand=False)
 
     def append(self, relpath, f):
         """Append the text in the file-like or string object to 
@@ -274,16 +281,17 @@
         # implementors don't have to implement everything.
         return self._iterate_over(relpaths, self.copy, pb, 'copy', expand=True)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
+        :param mode: This is the target mode for the newly created files
         TODO: This interface needs to be updated so that the target location
               can be different from the source location.
         """
         # The dummy implementation just does a simple get + put
         def copy_entry(path):
-            other.put(path, self.get(path))
+            other.put(path, self.get(path), mode=mode)
 
         return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
 

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2005-12-01 21:49:24 +0000
+++ bzrlib/transport/ftp.py	2005-12-18 04:08:46 +0000
@@ -179,11 +179,13 @@
         except ftplib.error_perm, e:
             raise NoSuchFile(self.abspath(relpath), extra=extra)
 
-    def put(self, relpath, fp):
+    def put(self, relpath, fp, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param f:       File-like or string object.
+        TODO: jam 20051215 This should be an atomic put, not overwritting files in place
+        TODO: jam 20051215 ftp as a protocol seems to support chmod, but ftplib does not
         """
         if not hasattr(fp, 'read'):
             fp = StringIO(fp)
@@ -194,7 +196,7 @@
         except ftplib.error_perm, e:
             raise TransportError(orig_error=e)
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         try:
             mutter("FTP mkd: %s" % self._abspath(relpath))

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-12-16 03:43:14 +0000
+++ bzrlib/transport/http.py	2005-12-18 04:08:46 +0000
@@ -196,7 +196,7 @@
                              % (self.abspath(relpath), str(e)),
                              orig_error=e)
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
@@ -204,7 +204,7 @@
         """
         raise TransportNotPossible('http PUT not supported')
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         raise TransportNotPossible('http does not support mkdir()')
 
@@ -218,7 +218,7 @@
         """Copy the item at rel_from to the location at rel_to"""
         raise TransportNotPossible('http does not support copy()')
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -232,7 +232,7 @@
         if isinstance(other, HttpTransport):
             raise TransportNotPossible('http cannot be the target of copy_to()')
         else:
-            return super(HttpTransport, self).copy_to(relpaths, other, pb=pb)
+            return super(HttpTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
 
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-12-16 04:42:07 +0000
+++ bzrlib/transport/local.py	2005-12-18 04:08:46 +0000
@@ -82,7 +82,7 @@
         except (IOError, OSError),e:
             self._translate_error(e, path)
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
@@ -93,7 +93,7 @@
         path = relpath
         try:
             path = self.abspath(relpath)
-            fp = AtomicFile(path, 'wb')
+            fp = AtomicFile(path, 'wb', new_mode=mode)
         except (IOError, OSError),e:
             self._translate_error(e, path)
         try:
@@ -114,12 +114,14 @@
             else:
                 yield relpath
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         path = relpath
         try:
             path = self.abspath(relpath)
             os.mkdir(path)
+            if mode is not None:
+                os.chmod(path, mode)
         except (IOError, OSError),e:
             self._translate_error(e, path)
 
@@ -162,7 +164,7 @@
             # TODO: What about path_to?
             self._translate_error(e, path)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -178,13 +180,17 @@
             for path in relpaths:
                 self._update_pb(pb, 'copy-to', count, total)
                 try:
-                    shutil.copy(self.abspath(path), other.abspath(path))
+                    mypath = self.abspath(path)
+                    otherpath = other.abspath(path)
+                    shutil.copy(mypath, otherpath)
+                    if mode is not None:
+                        os.chmod(otherpath, mode)
                 except (IOError, OSError),e:
                     self._translate_error(e, path)
                 count += 1
             return count
         else:
-            return super(LocalTransport, self).copy_to(relpaths, other, pb=pb)
+            return super(LocalTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
 
     def listable(self):
         """See Transport.listable."""

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2005-12-01 21:49:24 +0000
+++ bzrlib/transport/memory.py	2005-12-18 04:08:46 +0000
@@ -67,12 +67,12 @@
             raise NoSuchFile(relpath)
         return StringIO(self._files[relpath])
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """See Transport.put()."""
         self._check_parent(relpath)
         self._files[relpath] = f.read()
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """See Transport.mkdir()."""
         self._check_parent(relpath)
         if relpath in self._dirs:

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2005-12-16 03:43:14 +0000
+++ bzrlib/transport/sftp.py	2005-12-18 04:08:46 +0000
@@ -127,6 +127,13 @@
     def send(self, data):
         return os.write(self.proc.stdin.fileno(), data)
 
+    def recv_ready(self):
+        # TODO: jam 20051215 this function is necessary to support the
+        # pipelined() function. In reality, it probably should use
+        # poll() or select() to actually return if there is data
+        # available, otherwise we probably don't get any benefit
+        return True
+
     def recv(self, count):
         return os.read(self.proc.stdout.fileno(), count)
 
@@ -192,7 +199,8 @@
         self.lock_path = path + '.write-lock'
         self.transport = transport
         try:
-            self.lock_file = transport._sftp_open_exclusive(self.lock_path)
+            abspath = transport._abspath(self.lock_path)
+            self.lock_file = transport._sftp_open_exclusive(abspath)
         except FileExists:
             raise LockError('File %r already locked' % (self.path,))
 
@@ -343,32 +351,42 @@
             f.prefetch()
         return f
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """
         Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param f:       File-like or string object.
+        :param mode: The final mode for the file
         """
         final_path = self._abspath(relpath)
-        tmp_relpath = '%s.tmp.%.9f.%d.%d' % (relpath, time.time(),
+        self._put(final_path, f, mode=mode)
+
+    def _put(self, abspath, f, mode=None):
+        """Helper function so both put() and copy_abspaths can reuse the code"""
+        tmp_abspath = '%s.tmp.%.9f.%d.%d' % (abspath, time.time(),
                         os.getpid(), random.randint(0,0x7FFFFFFF))
-        tmp_abspath = self._abspath(tmp_relpath)
-        fout = self._sftp_open_exclusive(tmp_relpath)
-
+        fout = self._sftp_open_exclusive(tmp_abspath, mode=mode)
         closed = False
         try:
             try:
+                fout.set_pipelined(True)
                 self._pump(f, fout)
             except (IOError, paramiko.SSHException), e:
                 self._translate_io_exception(e, tmp_abspath)
+            if mode is not None:
+                self._sftp.chmod(tmp_abspath, mode)
             fout.close()
             closed = True
-            self._rename(tmp_abspath, final_path)
+            self._rename(tmp_abspath, abspath)
         except Exception, e:
             # If we fail, try to clean up the temporary file
             # before we throw the exception
             # but don't let another exception mess things up
+            # Write out the traceback, because otherwise
+            # the catch and throw destroys it
+            import traceback
+            mutter(traceback.format_exc())
             try:
                 if not closed:
                     fout.close()
@@ -389,13 +407,20 @@
             else:
                 yield relpath
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         try:
             path = self._abspath(relpath)
+            # In the paramiko documentation, it says that passing a mode flag 
+            # will filtered against the server umask.
+            # StubSFTPServer does not do this, which would be nice, because it is
+            # what we really want :)
+            # However, real servers do use umask, so we really should do it that way
             self._sftp.mkdir(path)
+            if mode is not None:
+                self._sftp.chmod(path, mode=mode)
         except (paramiko.SSHException, IOError), e:
-            self._translate_io_exception(e, relpath, ': unable to mkdir',
+            self._translate_io_exception(e, path, ': unable to mkdir',
                 failure_exc=FileExists)
 
     def _translate_io_exception(self, e, path, more_info='', failure_exc=NoSuchFile):
@@ -422,6 +447,9 @@
             # strange but true, for the paramiko server.
             if (e.args == ('Failure',)):
                 raise failure_exc(path, str(e) + more_info)
+            mutter('Raising exception with args %s', e.args)
+        if hasattr(e, 'errno'):
+            mutter('Raising exception with errno %s', e.errno)
         raise e
 
     def append(self, relpath, f):
@@ -442,7 +470,7 @@
         path_to = self._abspath(rel_to)
         self._copy_abspaths(path_from, path_to)
 
-    def _copy_abspaths(self, path_from, path_to):
+    def _copy_abspaths(self, path_from, path_to, mode=None):
         """Copy files given an absolute path
 
         :param path_from: Path on remote server to read
@@ -457,18 +485,13 @@
         try:
             fin = self._sftp.file(path_from, 'rb')
             try:
-                fout = self._sftp.file(path_to, 'wb')
-                try:
-                    fout.set_pipelined(True)
-                    self._pump(fin, fout)
-                finally:
-                    fout.close()
+                self._put(path_to, fin, mode=mode)
             finally:
                 fin.close()
         except (IOError, paramiko.SSHException), e:
             self._translate_io_exception(e, path_from, ': unable copy to: %r' % path_to)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -483,17 +506,11 @@
                 path_from = self._abspath(relpath)
                 path_to = other._abspath(relpath)
                 self._update_pb(pb, 'copy-to', count, total)
-                self._copy_abspaths(path_from, path_to)
+                self._copy_abspaths(path_from, path_to, mode=mode)
                 count += 1
             return count
         else:
-            return super(SFTPTransport, self).copy_to(relpaths, other, pb=pb)
-
-        # The dummy implementation just does a simple get + put
-        def copy_entry(path):
-            other.put(path, self.get(path))
-
-        return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
+            return super(SFTPTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
 
     def _rename(self, abs_from, abs_to):
         """Do a fancy rename on the remote server.
@@ -768,7 +785,7 @@
             pass
         return False
 
-    def _sftp_open_exclusive(self, relpath):
+    def _sftp_open_exclusive(self, abspath, mode=None):
         """Open a remote path exclusively.
 
         SFTP supports O_EXCL (SFTP_FLAG_EXCL), which fails if
@@ -779,19 +796,22 @@
         WARNING: This breaks the SFTPClient abstraction, so it
         could easily break against an updated version of paramiko.
 
-        :param relpath: The relative path, where the file should be opened
+        :param abspath: The remote absolute path where the file should be opened
+        :param mode: The mode permissions bits for the new file
         """
-        path = self._sftp._adjust_cwd(self._abspath(relpath))
+        path = self._sftp._adjust_cwd(abspath)
         attr = SFTPAttributes()
-        mode = (SFTP_FLAG_WRITE | SFTP_FLAG_CREATE 
+        if mode is not None:
+            attr.st_mode = mode
+        omode = (SFTP_FLAG_WRITE | SFTP_FLAG_CREATE 
                 | SFTP_FLAG_TRUNC | SFTP_FLAG_EXCL)
         try:
-            t, msg = self._sftp._request(CMD_OPEN, path, mode, attr)
+            t, msg = self._sftp._request(CMD_OPEN, path, omode, attr)
             if t != CMD_HANDLE:
                 raise TransportError('Expected an SFTP handle')
             handle = msg.get_string()
             return SFTPFile(self._sftp, handle, 'wb', -1)
         except (paramiko.SSHException, IOError), e:
-            self._translate_io_exception(e, relpath, ': unable to open',
+            self._translate_io_exception(e, abspath, ': unable to open',
                 failure_exc=FileExists)
 

